import gtk,  subprocess,  os
from PIL import Image


class ImageThing():

    def __init__(self):
        #initialise stuff
        
        #create temporary files for left and right images
        self.lfilename="/tmp/limagetemp.jpg"
        self.rfilename="/tmp/rimagetemp.jpg"
        #create gtk objects to hold thumbnails
        self.imageL=gtk.Image()
        self.imageR=gtk.Image()
        #container widget for side by side view
        hbox=gtk.HBox(False, 2)
        hbox.pack_start(self.imageR, True, False, 5)
        hbox.pack_start(self.imageL, True, False, 5)
        #image filter for later use
        self.filter = gtk.FileFilter()
        self.filter.set_name("Images")
        self.filter.add_mime_type("image/png")
        self.filter.add_mime_type("image/jpeg")
        self.filter.add_pattern("*.png")
        self.filter.add_pattern("*.jpg")
        #set up window
        self.window=gtk.Window()
        self.window.set_title("Stereo Image viewer thingy")
        self.window.set_size_request(1024, 450)
        self.window.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(100, 100, 100))

        # create a list of menu items, with accelerator keys and methods 
        filemenuitems=[("Open .MPO", "<Control>O", self.on_open), 
                       ("Open Left", "",  self.on_openL), 
                       ("Open Right","",  self.on_openR), 
                       ("Save",  "<Control>S",  self.on_save), 
                       ("Quit",  "<Control>Q", self.on_quit)]
        #create the menu bar
        menubar = gtk.MenuBar()
        mainmenu = gtk.Menu()
        filemenu = gtk.MenuItem("_File")
        filemenu.set_submenu(mainmenu)
        accelgroup = gtk.AccelGroup()
        self.window.add_accel_group(accelgroup)
        #iterate through list to generate menu items
        for item in filemenuitems:
            newitem=gtk.MenuItem(item[0])
            #add shortcuts keys if they exist
            if (item[1] != ""):
                key, mod = gtk.accelerator_parse(item[1])
                newitem.add_accelerator("activate", accelgroup, key,mod, gtk.ACCEL_VISIBLE)
            newitem.connect("activate", item[2])
            mainmenu.append(newitem)
        
        menubar.append(filemenu)
        
        #create a container for the main window and pack the menu into it
        
        vbox = gtk.VBox(False, 2)
        vbox.pack_start(menubar, False, False, 0)
        vbox.pack_start(hbox, True, False, 0)
        
        self.window.add(vbox)
        self.window.connect("destroy", gtk.main_quit)
        self.window.show_all()
        
        
        
    def on_open(self, item):
        #generate file dialog
        x=gtk.FileChooserDialog(title="Open a .MPO file...", \
                        action=gtk.FILE_CHOOSER_ACTION_OPEN, \
                        buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,\
                        gtk.STOCK_OK, gtk.RESPONSE_OK))
        #add a filter to show only .MPO files
        mpofilter=gtk.FileFilter()
        mpofilter.set_name("Stereo MPO files")
        mpofilter.add_pattern("*.MPO")
        x.add_filter(mpofilter)
        #get the response
        response=x.run()
        if response == gtk.RESPONSE_OK:
           self.get_mpo(x.get_filename())
        x.destroy()
        #load the image files
        pbuf=gtk.gdk.pixbuf_new_from_file_at_scale(self.lfilename, width=500, height=400, preserve_aspect_ratio=True)
        self.imageL.set_from_pixbuf(pbuf)
        #self.imageR.set_from_file(self.rfilename)
        pbuf=gtk.gdk.pixbuf_new_from_file_at_scale(self.rfilename, width=500, height=400, preserve_aspect_ratio=True)
        self.imageR.set_from_pixbuf(pbuf)
        
    def on_openL(self, item):
        #file dialog - same as above but for a single image
        x=gtk.FileChooserDialog(title="Open an image file for the Left...", \
                        action=gtk.FILE_CHOOSER_ACTION_OPEN, \
                        buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,\
                        gtk.STOCK_OK, gtk.RESPONSE_OK))
        x.add_filter(self.filter)
        response=x.run()
        if response == gtk.RESPONSE_OK:
           pbuf=gtk.gdk.pixbuf_new_from_file_at_scale(x.get_filename(),\
                                                       width=500, height=400,\
                                                       preserve_aspect_ratio=True)
           self.imageL.set_from_pixbuf(pbuf)
        x.destroy()
        
    def on_openR(self, item):
        # once again, a dialog but for a single image for the right
        x=gtk.FileChooserDialog(title="Open an image file for the Left...", \
                        action=gtk.FILE_CHOOSER_ACTION_OPEN, \
                        buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,\
                        gtk.STOCK_OK, gtk.RESPONSE_OK))
        x.add_filter(self.filter)
        response=x.run()
        if response == gtk.RESPONSE_OK:
           pbuf=gtk.gdk.pixbuf_new_from_file_at_scale(x.get_filename(),\
                                                       width=500, height=400,\
                                                       preserve_aspect_ratio=True)
           self.imageR.set_from_pixbuf(pbuf)
        x.destroy()
        
    def on_save(self, item):
        #generate save dialog
        x=gtk.FileChooserDialog(title="Save image as...", \
                        action=gtk.FILE_CHOOSER_ACTION_SAVE, \
                        buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,\
                        gtk.STOCK_OK, gtk.RESPONSE_OK))
        
        response=x.run()
        if response == gtk.RESPONSE_OK:
          # call the save pair method to actually process the files
          self.save_pair(x.get_filename())
        x.destroy()        
        
    def on_quit(self, item):
        self.window.destroy()
        gtk.main_quit()
        
    def get_mpo(self, fname):
    #exiftool -trailer:all= 3DPIC -o left.jpg
       if os.access(self.lfilename, os.F_OK) :
            os.remove(self.lfilename)
       cmd= "exiftool -trailer:all= "+fname+ " -o "+self.lfilename
       subprocess.check_call(cmd, shell=True)
       #TODO: add error checking!
       
    # exiftool 3DPIC.MPO -mpimage2 -b > right.jpg
       if os.access(self.rfilename, os.F_OK) :
            os.remove(self.rfilename)      
       cmd= "exiftool "+fname+ " -mpimage2 -b >"+self.rfilename
       subprocess.check_call(cmd, shell=True)   
       #TODO: add error checking!
    
    def save_pair(self, savename):
       #open the full images as PIL Image objects
       imageL = Image.open(self.lfilename)
       imageR = Image.open(self.rfilename)
       width,  height = imageL.size
       # create a new image to hold both
       buffer = Image.new("RGB", ((width*2)+(width/16), height))
       
       buffer.paste(imageR, (0, 0))
       # leave a gap when pasting to make it easier to see 
       buffer.paste(imageL, (width+(width/16), 0))
       #save the image
       buffer.save(savename, 'JPEG')
       return 0



def main():
    gtk.main()
    

if __name__ == "__main__":
    ImageThing()
    main()
